=begin
Copyright 2011, Matt666
All Rights Reserved

THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

License:   		Free

Author:     	Matthieu NOBLET
Organization:   Matt666
Name:      		Polarline
Version:   		1.1
SU Version:   	6 til 8
Date:      		25/05/2011
Description:   	Creat a line using polar mode
Usage:      	Type "Matt_Tools.polarline" in the ruby console to load it manually.
History:
	1.100	25/05/2011
			Some bugs resolved
			supports organizer
			Encapsulation

	1.000   04/07/2008
			First version
=end


class PolarLine

def initialize
	@ip = nil
    @ip1 = nil
	lg = nil
	ang = nil
	pl = nil
end

def reset(view)
	return @ip1.position
    @ip1.clear
    if( view )
        view.tooltip = nil
        view.invalidate if @drawn
    end
end

def activate
	model = Sketchup.active_model
	msg='Polar Line'
	Sketchup.version.to_i>=7 ? model.start_operation(msg,true) : model.start_operation(msg)
	@ip = Sketchup::InputPoint.new
    @ip1 = Sketchup::InputPoint.new
    Sketchup::set_status_text("Specify first point : ", SB_PROMPT)
    self.reset(nil)
end

def deactivate(view)
    view.invalidate if @drawn
end

def onCancel(flag, view)
	Sketchup.active_model.commit_operation
    self.reset(view)
end

def onMouseMove(flags, x, y, view)
	@ip.pick view, x, y
	if( @ip != @ip1 )
		view.invalidate if( @ip.display? or @ip1.display? )
		@ip1.copy! @ip
		view.tooltip = @ip1.tooltip
	end
end

def onLButtonDown(flags, x, y, view)
	@ip1.pick view, x, y
	if( @ip1.valid? )
		self.create_geometry(@ip1.position,view)
		self.reset(view)
	end
	view.lock_inference
end

def draw(view)
	self.draw_geometry(@ip1.position, view) if (@ip1.valid?) and (@ip1.display?)
end

def create_geometry(p1, view)
    ents = Sketchup.active_model.active_entities
    ents.add_cpoint(p1)
	n = ents.count
	Sketchup.send_action "selectSelectionTool:"
	lst =["","","Red-Green|Red-Blue|Green-Blue"]
	prompt = ["Length: ","trigo angle: ","3D plan: "]
	values = [$lg,$ang,$pl]
	result = inputbox(prompt,values,lst,"Polar parameters")
	if result
		$lg = result[0];$ang = result[1];$pl = result[2]
		puts("First point: " + p1.to_s + "\nLength: " + result[0].to_s + "\nAngle: " + result[1].to_s + "\nPlan: " + result[2])
		case Sketchup.active_model.options["UnitsOptions"]["LengthUnit"]
			when 0;lg = result[0].to_f
			when 1;lg = result[0].to_f.feet
			when 2;lg = result[0].to_f.mm
			when 3;lg = result[0].to_f.cm
			when 4;lg = result[0].to_f.m
		end
		ang = result[1].to_f.degrees
		pl = result[2]
		case pl
			when "Red-Green"
			x = p1[0] + lg * Math.cos(ang)
			y = p1[1] + lg * Math.sin(ang)
			z = p1[2]
			when "Red-Blue"
			x = p1[0] + lg * Math.cos(ang)
			y = p1[1]
			z = p1[2] + lg * Math.sin(ang)
			when "Green-Blue"
			x = p1[0]
			y = p1[1] + lg * Math.cos(ang)
			z = p1[2] + lg * Math.sin(ang)
		end
		p2 = [x,y,z]
		ents[n-1].erase!
		ents.add_edges(p1,p2) if p1 and p2
	else
		ents[n-1].erase!
		return nil
	end
	view.invalidate
end

def draw_geometry(p1, view)
    view.draw_points(p1)
	view.invalidate
end


end

module Matt_Tools
def self.polarline
    Sketchup.active_model.select_tool PolarLine.new
end
end #module

require 'sketchup.rb'
#Supports organizer.rb
unless file_loaded?(__FILE__)
	file_loaded(__FILE__)
	$submenu ? (organizer = $submenu) : (organizer = UI.menu("Draw"))
	organizer.add_item("Polar line"){ Matt_Tools.polarline }
	
	tb = UI::Toolbar.new("PolarLine")
	plug_dir=File.dirname(__FILE__), 'polarline'
	cmd = UI::Command.new("PolarLine") { Matt_Tools.polarline }
		cmd.large_icon = File.join(plug_dir, 'polarline24.png')
		cmd.small_icon = File.join(plug_dir, 'polarline16.png')
		cmd.tooltip = cmd.status_bar_text = 'Polar line'
		tb.add_item(cmd)
		tb.restore
end